//***************************************************************
//------------<27.Jan.2011 by Peter Reiser>----------------------
//***************************************************************

#ifndef MCU_H_
#define MCU_H_

//===============================================================

#include "trf796xBoosterPack.h"

//===============================================================


//
// MACRO-like functions
//
/******************************************************************************
 * @brief Writes a single register
 * @param a Register address
 * @param b Pointer to Buffer with data
 * @return  none
 *****************************************************************************/
#define RADIO_WRITESINGLE(a, b)     Radio_Transfer((a&0x1F), b, 1)
/******************************************************************************
 * @brief Reads a single register
 * @param a Register address
 * @param b Pointer to Buffer with data to send and where data will be stored
 * @return  none
 *****************************************************************************/
#define RADIO_READSINGLE(a, b)      Radio_Transfer(((a&0x1F)|BIT6), b, 1)
/******************************************************************************
 * @brief Writes continous registers
 * @param a Base Register address
 * @param b Pointer to Buffer with data
 * @param l lentgh
 * @return  none
 *****************************************************************************/
#define RADIO_WRITECONT(a, b, l)    Radio_Transfer(((a&0x1F)|BIT5), b, l)
/******************************************************************************
 * @brief Reads continous registers
 * @param a Register address
 * @param b Pointer to Buffer with data to send and where data will be stored
 * @param l lentgh
 * @return  none
 *****************************************************************************/
#define RADIO_READCONT(a, b, l)     Radio_Transfer(((a&0x1F)|(BIT5|BIT6)), b, l)
/******************************************************************************
 * @brief Sends a command to the radio
 * @param a Register address
 * @param b Pointer to Buffer with additional data
 * @return  none
 *****************************************************************************/
#define RADIO_DIRECTCOMMAND(a, b)    Radio_Transfer(((a&0x1F)|BIT7), b, 0)
/******************************************************************************
 * @brief Performs a raw transfer 
 * @param a Register address including mode and format
 * @param b Pointer to Buffer with data to send and where data will be stored
 * @param l lentgh
 * @return  none
 *****************************************************************************/
#define RADIO_WRITERAW(a, b, l)     Radio_Transfer(a, b, l)

//
// nfcid0 = 0x08 - dynamically generated by the NFC Forum Device (Length MUST be 4 bytes) (NFC Digital Protocols Tech Spec Req. 20)
//
#define TARGET_CID   {0x08, 0x01, 0x02, 0x03}

//#define TARGET_CID   {0x99, 0xA8, 0x62, 0x87} 		// 00 00 00 00 : App Info
// TMS3793 02 82 17 92 : App Info 00 92 EC 00	: App Data
// AMEX	   2A B6 19 86 : App Info 20 38 19 11 :
// Master Card 5E A4 08 46 : App Protocol 00 00 00 00

//#define TARGET_CID	 {0x08, 0x12, 0x34, 0x56, 0xED, 0x25,0x80}
extern u08_t TargetCID[7];

extern void Radio_Transfer(u08_t mode_addr, u08_t *pbuf, u08_t length);

void McuDelayMillisecond(u32_t n_ms);
void McuOscSel(u08_t mode);
void McuCounterSet(void);

//===============================================================

#endif
