//***************************************************************
//------------<02.Dec.2010 by Peter Reiser>----------------------
//***************************************************************

#ifndef _TRF796X_H_
#define _TRF796X_H_

//===============================================================

#include <msp430.h>                         // prozessor spezific header
#include <stdlib.h>                         // general purpose standard library
#include "mcu.h"
#include "spi.h"
#include "types.h"

extern u08_t buf[130];

//*****************************************************************************
//
// Card Emulation Select (Type A or Type B) 1 = Enable 0 = Disable
//
#define CARD_EMULATION_TYPE_A		1
#define CARD_EMULATION_TYPE_B		0
//*****************************************************************************

//*****************************************************************************
//
// Check for correct definition of RF_DAUGTHER_TRF79X0
//
#if (CARD_EMULATION_TYPE_A && CARD_EMULATION_TYPE_B)
#error "Only one card emulation mode can be defined at the same time."
#elif (!(CARD_EMULATION_TYPE_A || CARD_EMULATION_TYPE_B))
#error "Define the card emulation mode the TRF7970A should be configured as."
#endif

//==== TRF796x definitions ======================================

//---- Direct commands ------------------------------------------

#define IDLE                        0x00
#define SOFT_INIT                   0x03
#define RESET                       0x0F
#define TRANSMIT_CRC                0x11
#define	TRANSMIT_NO_CRC				0x10
#define TRANSMIT_NEXT_SLOT          0x14
#define STOP_DECODERS               0x16
#define RUN_DECODERS                0x17

//---- Reader register ------------------------------------------

#define CHIP_STATE_CONTROL              0x00
#define ISO_CONTROL                     0x01
#define ISO_14443B_OPTIONS              0x02
#define RX_WAIT_TIME					0x08
#define MODULATOR_CONTROL               0x09
#define RX_SPECIAL_SETTINGS             0x0A
#define REGULATOR_CONTROL               0x0B
#define IRQ_STATUS                      0x0C    // IRQ Status Register
#define IRQ_MASK                        0x0D    // Collision Position and Interrupt Mask Register
#define FIFO_IRQ_LEVEL					0x14
#define NFC_LOW_DETECTION	            0x16
#define NFCID                           0x17
#define NFC_TARGET_LEVEL                0x18
#define NFC_TARGET_PROTOCOL             0x19
#define NFC_TEST_SETTING1_REG			0x1A
#define FIFO_STATUS                     0x1C
#define TX_LENGTH_BYTE_1                0x1D

#define FIFO                            0x1F

//===============================================================

void Trf796xCommunicationSetup(void);
void Trf796xDirectCommand(u08_t *pbuf);
void Trf796xDirectMode(void);
void Trf796xDisableSlotCounter(void);
void Trf796xEnableSlotCounter(void);
void Trf796xInitialSettings(void);
void Trf796xRawWrite(u08_t *pbuf, u08_t length);
void Trf796xReConfig(void);
void Trf796xReadCont(u08_t *pbuf, u08_t length);
void Trf796xReadIrqStatus(u08_t *pbuf);
void Trf796xReadSingle(u08_t *pbuf, u08_t length);
void Trf796xReset(void);
void Trf796xResetIrqStatus(void);
void Trf796xRunDecoders(void);
void Trf796xStopDecoders(void);
void Trf796xTransmitNextSlot(void);
void Trf796xTurnRfOff(void);
void Trf796xTurnRfOn(void);
void Trf796xWriteCont(u08_t *pbuf, u08_t length);
void Trf796xWriteIsoControl(u08_t iso_control);
void Trf796xWriteSingle(u08_t *pbuf, u08_t length);

//===============================================================

void Radio_InitialSettings(void);
void Trf796xReaderSettings(void);

#endif
